/***************************** LICENSE START ***********************************

 Copyright 2012 ECMWF and INPE. This software is distributed under the terms
 of the Apache License version 2.0. In applying this license, ECMWF does not
 waive the privileges and immunities granted to it by virtue of its status as
 an Intergovernmental Organization or submit itself to any jurisdiction.

 ***************************** LICENSE END *************************************/

#include "MvQFolderWatcher.h"

#include <QDebug>
#include <QTimer>

#include "Folder.h"
#include "FolderPresenter.h"
 
QList<MvQFolderWatcher*> MvQFolderWatcher::items_;	
QMap<Folder*,time_t> MvQFolderWatcher::timeoutLog_;

MvQFolderWatcher::MvQFolderWatcher(FolderPresenter *fp,int timeout) :
 presenter_(fp),
 timeout_(timeout)
{
  	timer_= new QTimer(this);
	timer_->setInterval(timeout_);
	
	connect(timer_,SIGNAL(timeout()),
		this,SLOT(slotTimeout()));
		
	timer_->start();	 
}

Folder* MvQFolderWatcher::folder() 
{
	return presenter_->currentFolder();
}	

void MvQFolderWatcher::doTimeout(bool forced)
{
  	Folder *f=folder();
  	if(f && (forced || isTimeoutDue(time(0))))
	{
	  	timeoutLog_[f]=time(0);
		f->scan();
	}	
}  

bool MvQFolderWatcher::isTimeoutDue(time_t t)
{
  	QMap<Folder*,time_t>::iterator it=timeoutLog_.find(folder());
	if(it != timeoutLog_.end())
	{  
		//if more than the 80% of the timeout period passed!!
	  	return t  >= it.value()+8*timeout_/10000;
	}
	
	return true;
}

void MvQFolderWatcher::cleanLog()
{
	QList<Folder*> lst;
  	foreach(MvQFolderWatcher* item,items_)
	{  	
		lst << item->folder();
	}
		
	QList<Folder*> keys=timeoutLog_.keys();
	foreach(Folder* f,keys)
	{
		if(!lst.contains(f))  
			timeoutLog_.remove(f);
	}
}
	  

void MvQFolderWatcher::slotTimeout()
{
  	doTimeout(false);
}  

void MvQFolderWatcher::reload()
{
	timer_->stop();
	doTimeout(true);	
	timer_->start();
	
	print();
	
}  

void MvQFolderWatcher::add(FolderPresenter* fp)
{
	MvQFolderWatcher::
	cleanLog();
	
	MvQFolderWatcher *fw=new MvQFolderWatcher(fp);
	items_ << fw;
}	
	
void MvQFolderWatcher::remove(FolderPresenter* fp)
{
  	foreach(MvQFolderWatcher* item,items_)
	{
	  	if(item->folderPresenter() == fp)
		{
			items_.removeAll(item);
			delete item;
			return;
		}	
	}
}	

void MvQFolderWatcher::reload(FolderPresenter* fp)
{
	foreach(MvQFolderWatcher* item,items_)
	{
	  	if(item->folderPresenter() == fp)
		{
			item->reload();
			return;
		}	
	}  
}

void MvQFolderWatcher::print()
{
  	qDebug() << "Watched folders: \n----------------";
  	
  	foreach(MvQFolderWatcher* item,items_)
	{	
		qDebug() << QString::fromStdString(item->folder()->fullName()) << item->timeout()/1000 << "s";
	}	
}		
		