/***************************** LICENSE START ***********************************

 Copyright 2014 ECMWF and INPE. This software is distributed under the terms
 of the Apache License version 2.0. In applying this license, ECMWF does not
 waive the privileges and immunities granted to it by virtue of its status as
 an Intergovernmental Organization or submit itself to any jurisdiction.

 ***************************** LICENSE END *************************************/

#include "MvQSearchPanel.h"

#include <QHBoxLayout>
#include <QFormLayout>
#include <QLabel>
#include <QLineEdit>
#include <QListView> 
#include <QPainter>
#include <QPushButton>
#include <QStyle>
#include <QStyleOption>
#include <QToolButton>
#include <QVBoxLayout>

#include "FolderSearchData.h"
#include "MvQFolderModel.h"
#include "MvQLineEdit.h"
#include "MvQTheme.h"

//=======================================
//
// MvQFolderSearchModel:
//
//=======================================

MvQFolderSearchModel::MvQFolderSearchModel(QObject *parent) :  QSortFilterProxyModel(parent), folderModel_(0)
{
}

void MvQFolderSearchModel::setSourceModel(MvQFolderModel * sourceModel)
{
  	folderModel_=sourceModel;
	QSortFilterProxyModel::setSourceModel(sourceModel);
}

void MvQFolderSearchModel::slotFilterName(QString name)
{
  	name_=name.toStdString();
	reset();
}

void MvQFolderSearchModel::slotFilterType(QString type)
{
  	type_=type.toStdString();
	reset();
}

bool MvQFolderSearchModel::filterAcceptsRow(int sourceRow,
         const QModelIndex &/*sourceParent*/) const
{	
	if(!folderModel_)
		return false;
	
	if(name_.empty() && type_.empty())
	  	return true;
	
	QModelIndex index=sourceModel()->index(sourceRow,0);
	
	if(IconObject *obj=folderModel_->objectFromIndex(index))
	{
	  	return obj->match(name_,type_);
	
	}
	
	return false;
}	

//================================================
// 
//  MvQSearchPanel
//
//================================================

MvQSearchPanel::MvQSearchPanel(QWidget *parent) : 
      QWidget(parent)
{  
	QVBoxLayout *vb=new QVBoxLayout;
	vb->setSpacing(0); 
	vb->setContentsMargins(0,0,0,0);
	setLayout(vb);
	
	//Title + close button
	QHBoxLayout *hb=new QHBoxLayout;
	vb->addLayout(hb);
	
	hb->addSpacing(4);
	
	QLabel *label=new QLabel("<b>" + tr("Search") + "</b>",this);
	hb->addWidget(label);
	
	QToolButton *closeTb=new QToolButton(this);
	closeTb->setAutoRaise(true);	
	closeTb->setIcon(QPixmap(":/desktop/remove.svg")); 
	closeTb->setToolTip(tr("Close sidebar"));
	hb->addWidget(closeTb);
	
	connect(closeTb,SIGNAL(clicked(bool)),
		this,SIGNAL(closePanel(bool)));
						
	QWidget *searchW= new QWidget(this);
	QVBoxLayout *searchVb=new QVBoxLayout(searchW);	
	searchVb->setSpacing(0); 
	searchVb->setContentsMargins(1,1,1,1);
		
	QFormLayout *filterForm=new QFormLayout();
	nameLine_=new QLineEdit(searchW);
	typeLine_=new QLineEdit(searchW);
	filterForm->addRow(tr("Name:"),nameLine_);
	filterForm->addRow(tr("Type:"),typeLine_);
	searchVb->addLayout(filterForm);
	
	/*QHBoxLayout *nameHb=new QHBoxLayout;	
	label=new QLabel(tr("Filter:"));
	filterLine_=new QLineEdit(searchW);	
	nameHb->addWidget(label);
	nameHb->addSpacing(2);
	searchHb->addWidget(filterLine_,1);
	searchVb->addLayout(nameHb);*/
	
	
	searchVb->addSpacing(4);
	
	filterView_=new QListView(searchW);
	filterView_->setProperty("sidebarView","true");
	searchVb->addWidget(filterView_);
	
	vb->addWidget(searchW);
	
	model_=new MvQFolderModel(this); 
	model_->setIconSize(18);
	
	filterModel_=new MvQFolderSearchModel(this); 
	filterModel_->setSourceModel(model_);
	
	filterView_->setModel(filterModel_);
	
	//connect(filterList_,SIGNAL(itemActivated(QListWidgetItem*)),
	//	this,SLOT(slotIconSelected(QListWidgetItem*)));	
		
	connect(nameLine_,SIGNAL(textChanged(QString)),
		filterModel_,SLOT(slotFilterName(QString)));	
		
	connect(typeLine_,SIGNAL(textChanged(QString)),
		filterModel_,SLOT(slotFilterType(QString)));	
		
	connect(filterView_,SIGNAL(clicked(const QModelIndex&)),
		this,SLOT(slotSelected(const QModelIndex&)));	
		
	setProperty("sidebar","true");	
}


void MvQSearchPanel::setFolder(Folder *folder)
{ 		
	 model_->setFolder(folder);
}

void MvQSearchPanel::notInUse()
{  	
	if(model_->folder())
  		model_->setFolder(0);
}

void  MvQSearchPanel::slotSelected(const QModelIndex& index)
{
	if(IconObject* obj=model_->objectFromIndex(filterModel_->mapToSource(index)))
		emit iconSelected(obj);  
} 

void MvQSearchPanel::paintEvent(QPaintEvent *)
{
     QStyleOption opt;
     opt.init(this);
     QPainter p(this);
     style()->drawPrimitive(QStyle::PE_Widget, &opt, &p, this);
}
 
  
//================================================
// 
//  MvQFolderSearchPanel
//
//================================================

MvQFolderSearchPanel::MvQFolderSearchPanel(QWidget *parent) : 
      QWidget(parent)

{  
  	searchData_=new FolderSearchData;
	searchData_->setObserver(this);
  
  	QHBoxLayout *hb=new QHBoxLayout;
	hb->setSpacing(4); 
	hb->setContentsMargins(2,1,0,2);
	setLayout(hb);

	//Name
	QLabel *label=new QLabel(this);
	label->setText(tr("&Name:"));	
	nameLe_=new MvQLineEdit(this);
	nameLe_->setDecoration(QPixmap(":/desktop/search_decor.svg"));
	//nameLe_->setSizePolicy(QSizePolicy::Ignored, QSizePolicy::Preferred);
	//nameLe_->setMinimumWidth(100);
	
	label->setBuddy(nameLe_);
	
	hb->addWidget(label);	
	hb->addWidget(nameLe_,1);			
	
	//Type
	label=new QLabel(this);
	label->setText(tr("&Type:"));	
	typeLe_=new MvQLineEdit(this);
	typeLe_->setDecoration(QPixmap(":/desktop/search_decor.svg"));
	label->setBuddy(typeLe_);
	
	hb->addWidget(label);	
	hb->addWidget(typeLe_,1);	
	
	//Buttons	
	nextPb_ = new QPushButton(tr("&Next"),this);
	nextPb_->setIcon(QIcon(QPixmap(QString::fromUtf8(":/desktop/arrow_down.svg"))));

	prevPb_= new QPushButton(tr("&Previous"),this);
	prevPb_->setIcon(QIcon(QPixmap(QString::fromUtf8(":/desktop/arrow_up.svg"))));	
		
	hb->addWidget(nextPb_);
	hb->addWidget(prevPb_);
	
	hb->addStretch(1);
	
	connect(nameLe_, SIGNAL(textChanged(QString)), 
		this, SLOT(slotFind(QString)));

	connect(nameLe_, SIGNAL( returnPressed()), 
		this, SLOT(slotFindNext()));
				
	connect(typeLe_, SIGNAL( textChanged(QString)), 
		this, SLOT(slotFind(QString)));

	connect(typeLe_, SIGNAL( returnPressed()), 
		this, SLOT(slotFindNext()));
			
	connect(nextPb_, SIGNAL(clicked()), 
		this, SLOT(slotFindNext()));

	connect(prevPb_, SIGNAL(clicked()), 
		this, SLOT(slotFindPrev()));
		
	QToolButton *closeTb=new QToolButton(this);
	closeTb->setAutoRaise(true);	
	closeTb->setIcon(QPixmap(":/desktop/remove.svg")); 
	closeTb->setToolTip(tr("Close find panel"));
	hb->addWidget(closeTb);
	
	connect(closeTb,SIGNAL(clicked(bool)),
		this,SLOT(closePanel(bool)));
			
	oriColour_=nameLe_->palette().color(QPalette::Base);	
	notFoundColour_=MvQTheme::colour("search","not_found");
	foundColour_=MvQTheme::colour("search","found");
}

MvQFolderSearchPanel::~MvQFolderSearchPanel()
{
  	delete searchData_;
}	

void MvQFolderSearchPanel::slotFind(QString)
{
  	if(isVisible())
	{
	  	searchData_->setSearchTerm(nameLe_->text().toStdString(),typeLe_->text().toStdString());  
		emit find(searchData_);
		updateStatus();
		if(IconObject* obj=searchData_->first())
		  	emit iconSelected(obj); 
	}	
}

void MvQFolderSearchPanel::slotFindNext()
{
	if(isVisible())
	{  	
	  	if(IconObject* obj=searchData_->next())
	  		emit iconSelected(obj);  
	}	
}

void MvQFolderSearchPanel::slotFindPrev()
{
	if(isVisible())
	{
	  	if(IconObject* obj=searchData_->prev())
	  		emit iconSelected(obj);  	
	}	
}

void MvQFolderSearchPanel::targetChanged()
{
  	if(isVisible())
	{
	  	emit find(searchData_);
		updateStatus();
	}	
}

void MvQFolderSearchPanel::openPanel(bool)
{
	nameLe_->setFocus(Qt::OtherFocusReason);
	show();
	slotFind("");
}

void MvQFolderSearchPanel::closePanel(bool)
{	
  	hide();
	searchData_->clear();
	updateStatus();
	emit find(searchData_);	
}

void MvQFolderSearchPanel::matchChanged()
{
 	if(isVisible())
	{ 
	  
	}  
}  

void MvQFolderSearchPanel::updateStatus()
{
	QColor col=oriColour_;
	
	if(searchData_->isSet())
	{	  	
		if(!searchData_->hasMatch())
			col=notFoundColour_;
		else
		  	col=foundColour_;
	}
	
	QPalette p=nameLe_->palette();
	if(p.color(QPalette::Base) != col)
	{
	  	p.setColor(QPalette::Base,col);
		nameLe_->setPalette(p);
	}
	
	p=typeLe_->palette();
	if(p.color(QPalette::Base) != col)
	{	
	  	p.setColor(QPalette::Base,col);
		typeLe_->setPalette(p);
	}		
}

