/***************************** LICENSE START ***********************************

 Copyright 2012 ECMWF and INPE. This software is distributed under the terms
 of the Apache License version 2.0. In applying this license, ECMWF does not
 waive the privileges and immunities granted to it by virtue of its status as
 an Intergovernmental Organization or submit itself to any jurisdiction.

 ***************************** LICENSE END *************************************/

#ifndef SPECTRA_H
#define SPECTRA_H

#include <Metview.h>
//#include <MvFortran.h>
//#include <MvGribParC.h>

//#ifdef FORTRAN_UPPERCASE
//#define spectra_ SPECTRA
//#endif

//#ifdef FORTRAN_NO_UNDERSCORE
//#define spectra_ spectra
//#endif

//extern "C" void spectra_(void);

class SpectraF : public MvService {

public:

   SpectraF(char* a) : MvService(a) {}
   ~SpectraF() {}
   void serve(MvRequest&,MvRequest&);

   virtual int GetData(MvRequest&) = 0;
   int GetDataGen(MvRequest&);
   int ComputeSpectra(MvRequest&);
   virtual MvRequest PlotSpectra(MvRequest&) = 0;
   virtual void BuildAxis(MvRequest&,MvRequest&,MvRequest&) = 0;
   string GetTitle(MvRequest&);

protected :

   string      Stitle_;     // user title
   int         Stype_;      // SPECGRAPH or SPECCONTOUR
   int         Sdc_;        // subtracts out the diurnal cycle: TRUE or FALSE
   int         Snl24int_;   // plots only every 24 hours: TRUE or FALSE
   int         Sntmin_;     // axis lower value
   int         Sntlim_;     // axis upper value
   float       Smtrunc_;    // highest wavenumber in plots of spectra
   MvFieldSet  Sfset_;      // field set
   MvRequest   Svaxis_;     // vertical axis
};

class Specgraph : public SpectraF
{

public :

   Specgraph(char* a) : SpectraF(a) {}
   virtual int GetData(MvRequest&);
   virtual MvRequest PlotSpectra(MvRequest&);
   virtual void BuildAxis(MvRequest&,MvRequest&,MvRequest&);
   void BuildCurveLineParam(MvRequest&,int,int);

private:

   string    Stension_;     // interpolation for smoothing curves
   int       Ssmooth_;      // allows the curves to be smoothed: TRUE or FALSE
   MvRequest Shaxis_;       // horizontal axis
};

class Speccontour : public SpectraF
{

public :

   Speccontour(char* a) : SpectraF(a) {}
   virtual int GetData(MvRequest&);
   virtual MvRequest PlotSpectra(MvRequest&);
   virtual void BuildAxis(MvRequest&,MvRequest&,MvRequest&);

private:

   double Shilorad_;      // searchs radius for plotting of high/low markers
};

#endif
