/***************************** LICENSE START ***********************************

 Copyright 2012 ECMWF and INPE. This software is distributed under the terms
 of the Apache License version 2.0. In applying this license, ECMWF does not
 waive the privileges and immunities granted to it by virtue of its status as
 an Intergovernmental Organization or submit itself to any jurisdiction.

 ***************************** LICENSE END *************************************/

// MvTephiParams.cc - Nov2003/vk - original file
// ParamsThermo.cc  - Jan2014/fii - updated to new Thermo application

#include "ParamsThermo.h"

static const char* sPointSelection[] = { "AREA_AVERAGE", "COORDINATES", "STATION" };

MvThermoParams::MvThermoParams() :
   windFlag_(false),
   lat2_( 0.0 ),
   lon2_( 0.0 ),
   pointType_( eInterpolated )
{
}

MvThermoParams::~MvThermoParams()
{
}

bool MvThermoParams::init( const MvRequest& in )
{
   // Make sure that all parameters are defined
   MvRequest expReq = in;
   expReq = expReq.ExpandRequest("ThermoDataDef","ThermoDataRules",EXPAND_DEFAULTS);
expReq.print();

#if 0
  T1 = in("MINIMUM_TEMPERATURE");
  T2 = in("MAXIMAL_TEMPERATURE");
  P2 = in("BOTTOM_PRESSURE");
  P1 = in("TOP_PRESSURE");
  if( strcmp(in("BACKGROUND"),"ON") == 0 )
    backGround = 0;
#else
   T1 = -40;
   T2 = 40;
   P2 = 1020;
   P1 = 100; 
#endif

   // Retrieve dataset
   if ( !(int)expReq.countValues("DATA") ) {
      cout << "No Data files specified..." << endl;
      //setError(1, "No Data files specified...");
      return false;
   }
   expReq.getValue(dataReq_,"DATA");

   // Retrieve point selection
   string ps = (const char*)expReq("POINT_SELECTION");
   if ( ps == sPointSelection[0] )        //Area average
   {
      pointType_ = eAreaAverage;
      expReq.getValue( lat_,  ps.c_str(), 0 );  //-- n
      expReq.getValue( lon_,  ps.c_str(), 1 );  //-- w
      expReq.getValue( lat2_, ps.c_str(), 2 );  //-- s
      expReq.getValue( lon2_, ps.c_str(), 3 );  //-- e
   }
   else
   {
      // Retrieve point extraction
      string pe = (const char*)expReq("POINT_EXTRACTION");
      pointType_ = ( pe == "INTERPOLATE" ) ? eInterpolated : eGridPoint;
      if ( ps == sPointSelection[1] )
      {
         expReq.getValue( lat_,  ps.c_str(), 0 );
         expReq.getValue( lon_,  ps.c_str(), 1 );
      }
      else if ( ps == sPointSelection[2] )   //Station
      {
         MvRequest station;
         expReq.getValue( station, "STATION" );
         lat_ = station( "LATITUDE" );
         lon_ = station( "LONGITUDE" );
      }
   }

   // Retrieve dew point formulation
   dewPointFormulaType_ = (const char*)expReq("DEW_POINT_FORMULATION");

   // Retrieve fields parameter numbers
   uFieldId_  = (int)expReq( "U_WIND_PARAM" );
   vFieldId_  = (int)expReq( "V_WIND_PARAM" );
   tempId_    = (int)expReq( "TEMPERATURE_PARAM" );
   dewId_     = (int)expReq( "SPECIFIC_HUMIDITY_PARAM" );
   lnPressId_ = (int)expReq( "LNSP_PARAM" );

#if 0
  Wf = 0.;  //-- constant or a variable???????

   //-- check background params
  if( (T1 == T2) || (P1 == P2) )
    {
      ok_ = false;
      errorMsg_ = "Thermograms: Invalid Parameters (Temperature/Pressure)..";
      return;
    }
  if( T1 > T2 ) 
    {
      double tmp = T2;   T2 = T1;   T1 = tmp;
      errorMsg_ = "Thermograms: Minimun Temperature > Maximum...";
    }
  if( P1 > P2 )
    {
      double tmp = P2;   P2 = P1;   P1 = tmp;
      errorMsg_ = "Thermograms: Top Pressure > Bottom...";
    }

#endif
   return true;
}

string MvThermoParams::formulaPrintName()
{
   string name = dewPointFormulaType_;
   for( unsigned int it = 0; it < name.length(); ++it )
   {
      char cc = name[it];
      name[it] = ( cc == '_' ) ? ' ' : tolower(cc);
   }

   return name;
}
