/***************************** LICENSE START ***********************************

 Copyright 2012 ECMWF and INPE. This software is distributed under the terms
 of the Apache License version 2.0. In applying this license, ECMWF does not
 waive the privileges and immunities granted to it by virtue of its status as
 an Intergovernmental Organization or submit itself to any jurisdiction.

 ***************************** LICENSE END *************************************/

#include "Thermo.h"
#include "GribThermo.h"
#include "BufrThermo.h"

Thermo::Thermo (const char* kw) :
             MvService(kw),
             actionMode_("prepare")
{

}

void Thermo::serve(MvRequest& in,MvRequest& out)
{
cout << "request IN" << endl;
in.print();

   // Create a temporary netCDF file name
   ncFileName_ = marstmp();

   // Get and save action mode. Default value is "prepare"
   actionMode_ = (const char*)in("_ACTION") ? (const char*)in("_ACTION") : "prepare";

   // Get Input parameters
   //origReq_ = in;
   if ( !this->getInputParameters(in) )
      return;

   // Process data
   if ( !this->processData() )
      return;

   // Build the output request
   if ( !this->createOutputRequest(out) )
      return;

   // Add hidden values to tne output request
   // out1("_VERB") = "NETCDF_MATRIX";
   if ( (const char *)in("_NAME") )
      out("_NAME") = in("_NAME");

cout << "request OUT" << endl;
out.print();

   return;
}

bool Thermo::isVisualise()
{
   // If the output is not 'visualisation' related: actions
   // "not visualise" or "prepare" (in certain cases), then
   // send back the "netCDF" request
   #if 0
   int    procType   = appInfo.processType();
   if ( actionMode != "visualise" &&
        !(actionMode == "prepare" && procType == XS_DATA_MODULE_DROPPED) &&
        !(actionMode == "prepare" && procType == XS_DATA_DROPPED) )
      return false;
   #endif
   if ( actionMode_ != "visualise" )
      return false;
   else
      return true;
}

//------------------------------------------------------------------------------

int main(int argc,char **argv)
{
   MvApplication theApp(argc,argv);

   GribThermo gribThermo;
   BufrThermo bufrThermo;

   // The applications don't try to read or write from pool, this
   // should not be done with the new PlotMod.
   gribThermo.saveToPool(false);
   bufrThermo.saveToPool(false);

   theApp.run();
}
