/***************************** LICENSE START ***********************************

 Copyright 2014 ECMWF and INPE. This software is distributed under the terms
 of the Apache License version 2.0. In applying this license, ECMWF does not
 waive the privileges and immunities granted to it by virtue of its status as
 an Intergovernmental Organization or submit itself to any jurisdiction.

 ***************************** LICENSE END *************************************/

#include <QComboBox>
#include <QDebug>
#include <QDialogButtonBox>
#include <QGridLayout>
#include <QGroupBox>
#include <QHBoxLayout>
#include <QImageReader>
#include <QLabel> 
#include <QLineEdit>
#include <QMouseEvent>
#include <QMessageBox>
#include <QNetworkAccessManager>
#include <QNetworkReply>
#include <QNetworkRequest>
#include <QRadioButton>
#include <QPainter>
#include <QSettings>
#include <QVBoxLayout>
#include <QtCore/QTextStream>
#include <QFile>
#include <QMessageBox>

#include <boost/property_tree/ptree.hpp>
#include <boost/property_tree/json_parser.hpp>
#include <boost/foreach.hpp>

#include "MvQWeatherRoomDialog.h"

//#include "uPlot.h"
//#include "MvRequestUtil.hpp"

MvQVideoWallCellInfo::MvQVideoWallCellInfo(double x, double y, double w, double h, int number)
{
	x_ = x;
	y_ = y;
	w_ = w;
	h_ = h;
	number_ = number;
}


//===========================================
//
// MvQVideoWallInfo
//
//============================================

MvQVideoWallServer::MvQVideoWallServer(QObject* parent) : QObject(parent)
{
	QNetworkAccessManager *manager = new QNetworkAccessManager(this);
	connect(manager, SIGNAL(finished(QNetworkReply*)),
		 	this, SLOT(slotReply(QNetworkReply*)));


	URL_   = "http://eccharts-test.ecmwf.int/wall";    // default, should be in preferences
	token_ = "metview";                                // default, should be in preferences

	const char *URL = getenv ("METVIEW_WALL_URL");     // override if env var exists
	if (URL)
		URL_ = URL;

	const char *token = getenv ("METVIEW_WALL_TOKEN"); // override if env var exists
	if (token)
		token_ = token;

	
	//QString url("http://localhost:8001/wall");
	//QString token("key0");
	QString req= QString("%1/current-layouts/?key=%2").arg(URL_).arg(token_); 

	//http://localhost:8001/wall/setup/dummy?key=key0
	QNetworkRequest netReq(QUrl::fromPercentEncoding(req.toUtf8()));
	
	qDebug() << req;

	manager->get(netReq);	
}


bool MvQVideoWallServer::parseLayoutResponse(QString &response)
{

	// write the response to a text file
	QString tmpFilename(marstmp());
	QFile tmpFile(tmpFilename);
	tmpFile.open(QIODevice::WriteOnly);
 

	// did we manage to open the tmp file for writing?
	if (!tmpFile.isOpen())
	{
		QMessageBox::warning(0, QString("Metview"),
							 tr("Error, unable to open temporary file %1 to write server response")
							 .arg(tmpFilename));
		return false;
	}

	// write the server response to the file
	QTextStream outStream(&tmpFile);
	outStream << response;
	tmpFile.close();



	// parse the response using the boost JSON property tree parser
	using boost::property_tree::ptree;
	ptree pt;
	std::string tmpFilenameStd = tmpFilename.toUtf8().constData();

	try
	{
		read_json(tmpFilenameStd, pt);
	}
	catch (const boost::property_tree::json_parser::json_parser_error& e)
	{
		QMessageBox::warning(0, QString("Metview"),
							 tr("Error, unable to parse JSON response from server"));
		return false;
	}


	wallInfos_.clear();

	// for each layout
	for (ptree::const_iterator itOuter = pt.begin(); itOuter != pt.end(); ++itOuter)
	{
		ptree const &layout = itOuter->second;
		MvQVideoWallInfo wallInfo;

		for (ptree::const_iterator itLayout = layout.begin(); itLayout != layout.end(); ++itLayout)
		{
			int totalRows = 0; // must take into account the rowspans
			int totalCols = 0; // must take into account the colspans

			if (itLayout->first == "layout")
			{
				ptree const &layout = itLayout->second;

				// for each row
				int r = 0;
				for (ptree::const_iterator itRows = layout.begin(); itRows != layout.end(); ++itRows)
				{
					ptree const &row = itRows->second;

					// for each column in the current row
					int c=0;
					for (ptree::const_iterator itCols = row.begin(); itCols != row.end(); ++itCols)
					{
						int colSpan    = itCols->second.get<int>("colspan", 1);
						int rowSpan    = itCols->second.get<int>("rowspan", 1);
						
						if (r + rowSpan > totalRows)
							totalRows = r + rowSpan;

						if (c + colSpan  > totalCols)
							totalCols = c + colSpan;

						c += colSpan;
					}
					r += 1;
				}


				////////////////////////////////////

				//ptree const &layout = itLayout->second;
				std::string cname = itLayout->second.get("name", "no_name");
				int numRows = layout.size();

				double rowHeight = 1.0 / totalRows;
				double currentY  = 0.0;

				// for each row
				for (ptree::const_iterator itRows = layout.begin(); itRows != layout.end(); ++itRows)
				{
					ptree const &row = itRows->second;
					int numCols = row.size();
					//std::cout << "	num cols: " << numCols << std::endl;

					double colWidth = 1.0 / totalCols;
					double currentX = 0.0;

					// for each column in the current row
					for (ptree::const_iterator itCols = row.begin(); itCols != row.end(); ++itCols)
					{
						int cellNumber = itCols->second.get<int>("cell",   -1);
						
						if (cellNumber != -1)
						{
							int colSpan    = itCols->second.get<int>("colspan", 1);
							int rowSpan    = itCols->second.get<int>("rowspan", 1);

							MvQVideoWallCellInfo cellInfo(currentX, currentY, colWidth*colSpan, rowHeight*rowSpan, cellNumber);
							wallInfo.addCellInfo(cellInfo);

							// set up the x-coord for the next column
							currentX += colWidth*colSpan;
						}
						else
						{
							currentX += colWidth;
						}
					}

					// set up the y-coord for the next row
					currentY += rowHeight;
				}
			}
			else if (itLayout->first == "wall")
			{
				std::string cname = itLayout->second.get_value<std::string>("name");
				QString qname(cname.c_str());
				//std::cout << "	 name: " << cname << endl;
				wallInfo.setName(qname);
			}
		}

		wallInfos_.push_back(wallInfo);
	}


	//BOOST_FOREACH( ptree::value_type const &v, pt.get_child("") )
	//{
	//	std::string f = v.first;
	//	std::string s = v.second.get<std::string>("carrier");
	//	qDebug() << "another layout!\n" << QString(v.first.c_str());
	//}

	//print(pt, 0);
	
	return true;
}



void MvQVideoWallServer::slotReply(QNetworkReply* reply)
{
	QNetworkRequest r=reply->request();
	QUrl url=r.url();
  	
	QString mimeType=reply->header(QNetworkRequest::ContentTypeHeader).toString();
	
	qDebug() << "mimetype" << mimeType;
	
	if(reply->error() != QNetworkReply::NoError)
	{
		QMessageBox::warning(0, QString("Metview"),
				tr("Error, unable to get server response : ")
				.arg(reply->errorString()));
		
		//marslog(LOG_EROR,"Logo download failed!\nError: %s ",reply->errorString().toStdString().c_str());
		//log->error(reply->errorString().toStdString() + "\n");			
		//QApplication::exit();
		return;	
	}
	else
	{
	 	 QByteArray ba=reply->readAll();
		 QString res(ba);
		 parseLayoutResponse(res);

		 emit gotServerResponse();
	}  
	
	reply->deleteLater();
}



//===========================================
//
// MvQVideoWallInfo
//
//============================================

MvQVideoWallInfo::MvQVideoWallInfo()
{
}


//===========================================
//
// MvQVideoWallWidget
//
//============================================

MvQVideoWallWidget::MvQVideoWallWidget(QWidget *parent) : 
  QWidget(parent),
  offset_(1),
  current_(0)
{
  	setSizePolicy(QSizePolicy(QSizePolicy::Minimum, QSizePolicy::Minimum));
	setMinimumSize(QSize(128,128));
	
	setMouseTracking(true);
	
	int size=128;
	QImageReader imgR(":/uPlot/wall_screen.svg");
	if(imgR.canRead())
	{
		imgR.setScaledSize(QSize(size,size));
		QImage img=imgR.read();
		pix_=QPixmap::fromImage(img);
	}
	QImageReader imgRs(":/uPlot/wall_screen_selected.svg");
	if(imgRs.canRead())
	{
		imgRs.setScaledSize(QSize(size,size));
		QImage img=imgRs.read();
		pixSelect_=QPixmap::fromImage(img);
	}
}


void MvQVideoWallWidget::setWall(MvQVideoWallInfo *info)
{
	if (!info)
	{
		items_ << QRect(0,0,100,100); 
	}
	else
	{
		items_.clear();
		for (int i=0 ; i < info->cellInfos().size(); i++)
		{
			MvQVideoWallCellInfo *cell = &(info->cellInfos()[i]);
			items_ << QRect(cell->x()*100., cell->y()*100., cell->w()*100., cell->h()*100.);
		}
		setEnabled(true);
	}

	update();
}  

void MvQVideoWallWidget::paintEvent(QPaintEvent*)
{  
	int w=width()-2*offset_;
	int h=height()-2*offset_; 
	
	QPainter painter(this);
	
	if (!isEnabled())
	{
		QString message("Retrieving layout from server...");
		QTextOption textOptions;
		textOptions.setAlignment(Qt::AlignHCenter);
		textOptions.setWrapMode(QTextOption::WordWrap);
			
		painter.drawText(QRect(0,0,w,h), message, textOptions);
		return;
	}
		
		
	QColor col(128,128,255);
	QBrush brush;
	brush.setStyle(Qt::SolidPattern);
	
	painter.setPen(Qt::black);
	
  	for(int i=0; i < items_.count(); i++)
	{
	  	QRectF r(offset_+(w*items_[i].x())*0.01,offset_+(h*items_[i].y())*0.01,
			 (w*items_[i].width())*0.01,(h*items_[i].height())*0.01);

		//qDebug() << i << items_[i].x() << items_[i].y() << items_[i].width() << items_[i].height() << r; 		 
		
		
		if(i==current_)
			painter.drawPixmap(r,pixSelect_,QRectF(0,0,pixSelect_.width(),pixSelect_.height()));  	
		else
		  	painter.drawPixmap(r,pix_,QRectF(0,0,pix_.width(),pix_.height()));

	}	 
}  


void MvQVideoWallWidget::updateSelection(const QString &selection)
{
	int screenNum = selection.toInt();
	current_ = screenNum;
	update();
}


void MvQVideoWallWidget::mousePressEvent(QMouseEvent *event)
{	
	if(select(event->pos()))
	{
		update();
		QString newSelection;
		newSelection.setNum(current_);
		emit selectionChanged(newSelection);
	}	 
}

bool MvQVideoWallWidget::select(QPoint pos)
{
	int x=(pos.x()-offset_)*100/width();
	int y=(pos.y()-offset_)*100/height();
	
	for(int i=0; i < items_.count(); i++)
	{
	  	if(items_.at(i).contains(QPoint(x,y)))
		{
		  	current_=i;
			return true;
		}
	}	

	return false;
}

//===========================================
//
// MvQWeatherRoomDialog
//
//============================================

MvQWeatherRoomDialog::MvQWeatherRoomDialog(int currentFrame, int totalFrames, QWidget *parent) :
	QDialog(parent),
	current_(currentFrame),
	total_(totalFrames)
{	
	QVBoxLayout* vb=new QVBoxLayout();
	setLayout(vb);
	
	//Wall combo
	QLabel *label=new QLabel(tr("Video wall:"),this);
	wallCb_=new QComboBox(this);
	QHBoxLayout* hb=new QHBoxLayout();
  	hb->addWidget(label);
	hb->addWidget(wallCb_,1);
	vb->addLayout(hb);
	
	wallServer_= new MvQVideoWallServer(this);
	connect(wallServer_, SIGNAL(gotServerResponse()), this, SLOT(updateWallSelector()));
	connect(wallCb_, SIGNAL(currentIndexChanged(int)), this, SLOT(updateWallSelection(int)));

	wallCb_->addItem("No wall");
	
	//Screen selector
	screenSelector_= new MvQVideoWallWidget(this);


	vb->addWidget(screenSelector_,1);
	
	screenSelector_->setWall(NULL);
	screenSelector_->setEnabled(false);

	// screen selection text box (for if the layout plot is not sufficient)
	QHBoxLayout* hb2 = new QHBoxLayout();
	QLabel *screenSelectionLabel = new QLabel(tr("Screen:"),this);
	hb2->addWidget(screenSelectionLabel);
	leScreenSelection_ = new QLineEdit(this);
	QValidator *intValidator = new QIntValidator(100, 999, this);
	leScreenSelection_->setValidator(intValidator);
	hb2->addWidget(leScreenSelection_);
	vb->addLayout(hb2);

	// two-way connection between the screen selection widget and text box
	connect(leScreenSelection_, SIGNAL(textEdited(const QString &)),
		screenSelector_, SLOT(updateSelection(const QString &)));

	connect(screenSelector_, SIGNAL(selectionChanged(const QString &)),
		leScreenSelection_, SLOT(setText(const QString &)));

	leScreenSelection_->setText("0");  // initialise the text box


	//Frame selection
	buildFrameSelection(vb);
  	
	// Buttonbox
	QDialogButtonBox* buttonBox= new QDialogButtonBox(this);
	
	buttonBox->addButton(QDialogButtonBox::Ok);
	buttonBox->addButton(QDialogButtonBox::Cancel);	
	
	connect(buttonBox, SIGNAL(accepted()), this, SLOT(accept()));
   connect(buttonBox, SIGNAL(rejected()), this, SLOT(reject()));

	vb->addWidget(buttonBox);	
	
	//Read the previous gui settings
	readSettings();		
}

// Destructor
MvQWeatherRoomDialog::~MvQWeatherRoomDialog()
{
	writeSettings();
}

void MvQWeatherRoomDialog::setVisible(bool b)
{
	QDialog::setVisible(b);
}

void MvQWeatherRoomDialog::updateWallSelection(int wallIndex)
{
	if ((wallIndex > -1) && (wallServer_->numWalls() > 0))
		screenSelector_->setWall(wallServer_->wallInfo(wallIndex));
}



void MvQWeatherRoomDialog::updateWallSelector()
{
	MvQVideoWallInfo *wallInfo;

	// update the list of walls

	wallCb_->clear();

	for (size_t i = 0; i < wallServer_->numWalls(); i++)
	{
		wallInfo = wallServer_->wallInfo(i);
		QString fullWallName = QString("Wall %1 : %2").arg(i+1).arg(wallInfo->name());
		wallCb_->addItem(fullWallName);
	}

	screenSelector_->setWall(wallServer_->wallInfo(0)); // by default, choose the first wall
}

void MvQWeatherRoomDialog::slotRangeFrameSelection(bool checked)
{
	// Enable/Disable line edit
	//leFs3_->setEnabled(checked);
}

void MvQWeatherRoomDialog::buildFrameSelection(QLayout *layout)
{
	// Group Box with radio buttons and one line edit
	QGroupBox* gbFs = new QGroupBox( tr("Frame Selection"),this );
	cbFs1_ = new QRadioButton( tr("&Current"), gbFs);
	cbFs2_ = new QRadioButton( tr("&All"), gbFs );
	cbFs3_ = new QRadioButton( tr("&Range"), gbFs );
	cbFs1_->setChecked(true);

	// Line edit with a validator
	leFs3_ = new QLineEdit(this);
	leFs3_->setToolTip(tr("Specify one or more frame ranges, e.g. 1-3,6,7"));
	leFs3_->setEnabled(false);
	QRegExp rx("^\\d{1,}(-\\d{1,})?(,\\d{1,}|,\\d{1,}-\\d{1,})*");
	QValidator *validator = new QRegExpValidator(rx, this);
	leFs3_->setValidator(validator);

	// Set layout
	QGridLayout* glFs = new QGridLayout( gbFs );
	glFs->addWidget( cbFs1_, 0, 0 );
	glFs->addWidget( cbFs2_, 1, 0 );
	glFs->addWidget( cbFs3_, 2, 0 );
	glFs->addWidget( leFs3_, 2, 1 );

	connect(cbFs3_,SIGNAL(toggled(bool)), this, SLOT(slotRangeFrameSelection(bool)));

	// Attach widget to the parent
	layout->addWidget(gbFs);

}

bool MvQWeatherRoomDialog::getFrameSelection( MvRequest& req )
{
	// Three options: CURRENT, ALL, RANGE
	// ALL: nothing need to be done
	/*if ( cbFs2_->isChecked() )
		return true;

	// CURRENT: get the current frame numbeer
	int i;
	QList<int> list;
	if ( cbFs1_->isChecked() )
	{
		list.append(current_+1);  // start from 1, not from 0
	}
	else	// RANGE
	{
		// Get the list of range entries (split by comma)
		QStringList list1 = leFs3_->text().split(",", QString::SkipEmptyParts);

		// Analyse all entries
		for ( i = 0; i < list1.size(); i++)
		{
			// Check if it is a single value or a range of values
			QStringList list2 = list1.at(i).split("-", QString::SkipEmptyParts);
			if ( list2.size() == 1 ) // single value
				list.append(list2.at(0).toInt());
			else
			{
				int i1 = list2.at(0).toInt();
				int i2 = list2.at(1).toInt();
				if ( i1 > i2 )  // invert sequence
				{
					int iaux = i1;
					i1 = i2;
					i2 = iaux;
				}
				for ( int j = i1; j <= i2; j++ )
					list.append(j);
			}
		}

		// Check if the range of values is valid
		qSort(list.begin(), list.end());
		if ( list.at(list.size()-1) > total_ )
		{
			ostrstream info;
			info << "Enter values from 1 to " << total_ << ends;
			errorMessage("Invalid range!",(const char*)info.str());
			return false;
		}
	}

	// Save list of frames selection for all drivers
	while ( req )
	{
		for ( i = 0; i < list.size(); i++)
			req.addValue("OUTPUT_FRAME_LIST",list.at(i));

		req.advance();
	}

	req.rewind();*/

	return true;
}

void MvQWeatherRoomDialog::errorMessage (const char* msg, const char* info)
{
	/*QMessageBox msgBox;
	msgBox.setIcon(QMessageBox::Critical);
	msgBox.setWindowTitle(tr("Export Error"));
	msgBox.setText(QString::fromUtf8(msg));
	if ( info )
		msgBox.setInformativeText(QString::fromUtf8(info));

	msgBox.exec();*/
}

void MvQWeatherRoomDialog::readSettings()
{
	// Restore previous gui settings
	QSettings settings("ECMWF","uPlot_MvQWeatherRoomDialog");
	
	settings.beginGroup("main");
	if(settings.contains("size"))
	{  
		resize(settings.value("size").toSize());
	}
	else
	{
	  	resize(QSize(420,400));
	}
	
	settings.endGroup(); 
}
	

void MvQWeatherRoomDialog::writeSettings()	
{	
	// Save current gui settings
	QSettings settings("ECMWF","uPlot_MvQWeatherRoomDialog");
	
	settings.beginGroup("main");
	settings.setValue("size",size());
	settings.endGroup();
}
